<?

class http {
	const MULTIPART = "multipart/form-data";
	const ATOMXML = "application/atom+xml";
	
	public $proxy = null;
	public $proxy_type = CURLPROXY_HTTP;
	public $timeout = null;
	public $follow_location = true;
	public $referer = null;
	public $custom_headers = null;
	public $cookies = null;
	public $user_agent = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)";
	
	public $encoding = null;
	public $current_url = null;
	public $headers = array();
	
	private $curl = null;
	private $mhcurl = null;
	private $mcurl = array();
	
	function GET($url,$data=null) {
		if (!is_null($data)) {
			$url = parse_url($url);
			$query = array();
			if (isset($url['query'])) {
				parse_str($url['query'],$query);
			}
			$query = $data + $query;
			$url['query'] = http_build_query($query);
			$url = self::buildURL($url);
		}
		if (is_array($url)) {
			$this->multi_init($url);
			return $this->processMultiRequest();
		} else {
			$this->init($url);
			return $this->processRequest();
		}
	}
	
	function POST($url,$data,$type=null) {
		$this->init($url);
		if (!is_null($type)) {
			curl_setopt($this->curl, CURLOPT_HTTPHEADER, array("Content-Type: ".$type));
		}
		curl_setopt($this->curl, CURLOPT_POST, true);
		curl_setopt($this->curl, CURLOPT_POSTFIELDS, http_build_query($data));
		return $this->processRequest();
	}
	
	private function multi_init($urls) {
		$this->mhcurl = curl_multi_init();
		$this->mcurl = array();
		foreach ($urls as $i => $url) {
			$this->mcurl[$i] = $this->init($url);
		    curl_multi_add_handle($this->mhcurl, $this->mcurl[$i]);
		}
	}
	
	private function init($url) {
		$this->current_url = $url;
		$this->curl = curl_init($url);
		curl_setopt($this->curl, CURLOPT_URL, $url);
		if (parse_url($url,PHP_URL_SCHEME) == "https") {
			curl_setopt($this->curl, CURLOPT_SSL_VERIFYHOST, 0);
			curl_setopt($this->curl, CURLOPT_SSL_VERIFYPEER, 0);		
		}
		if (!is_null($this->timeout)) {
			curl_setopt($this->curl, CURLOPT_TIMEOUT, $this->timeout);
		}
		if (!is_null($this->referer)) {
			curl_setopt($this->curl, CURLOPT_REFERER, $this->referer);
		}
		if (!is_null($this->custom_headers)) {
			curl_setopt($this->curl, CURLOPT_HTTPHEADER, $this->custom_headers);
		}
		if (!is_null($this->cookies)) {
			curl_setopt($this->curl, CURLOPT_COOKIE, $this->processCookies($this->cookies));
		}
		if (!is_null($this->proxy)) {
			curl_setopt($this->curl, CURLOPT_PROXYTYPE, $this->proxy_type);
			curl_setopt($this->curl, CURLOPT_PROXY, $this->proxy);
		}
		curl_setopt($this->curl, CURLOPT_USERAGENT, $this->user_agent);
		curl_setopt($this->curl, CURLOPT_FOLLOWLOCATION, $this->follow_location);
	    curl_setopt($this->curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($this->curl, CURLOPT_HEADER, true);
		return $this->curl;
	}
	
	private function processMultiRequest() {
		$result = array();
		$running = null;
		do {
			curl_multi_exec($this->mhcurl, $running);
		} while($running);
		foreach($this->mcurl as $curl) {
			$headers_size = curl_getinfo($curl,CURLINFO_HEADER_SIZE);
			$result[] = substr(curl_multi_getcontent($curl),$headers_size);
			curl_multi_remove_handle($this->mhcurl, $curl);
		}
		curl_multi_close($this->mhcurl);
		return $result;
	}
	
	private function processRequest() {
		$result = curl_exec($this->curl);
		$headers_size = curl_getinfo($this->curl,CURLINFO_HEADER_SIZE);
		$headers = substr($result,0,$headers_size);
		$this->processHeaders($headers);
		$this->current_url = curl_getinfo($this->curl,CURLINFO_EFFECTIVE_URL);
		$this->referer = $this->current_url;
		curl_close($this->curl);
		return substr($result,$headers_size);
	}
	
	private function processCookies($cookies) {
		$result = "";
		foreach ($cookies as $key => $value) {
			$result .= urlencode($key)."=".urlencode($value).";";
		}
		return substr($result,0,-1);
	}
	
	private function processHeaders($headers) {
		$this->headers = array();
		foreach(explode("\r\n",$headers) as $header) {
			if (strpos($header,":") !== false) {
				list($key,$value) = explode(":",$header,2);
				$key = trim($key);
				$value = trim($value);
				if (isset($this->headers[$key])) {
					if (is_array($this->headers[$key])) {
						$this->headers[$key][] = $value;
					} else {
						$this->headers[$key] = array($value);
					}
				} else {
					$this->headers[strtolower($key)] = $value;
				}
			}
		}
		if (isset($this->headers['set-cookie'])) {
			foreach ((array)$this->headers['set-cookie'] as $value) {
				if (($pos = strpos($value,";")) !== false) $value = substr($value,0,$pos);
				if (strpos($value,"=") !== false) {
					list($key,$value) = explode("=",$value,2);
				} else {
					$key = $value;
					$value = "";
				}
				$key = trim($key);
				$value = trim($value);
				$this->cookies[$key] = $value;
			}
		}
		$this->encoding = null;
		if (isset($this->headers['content-type'])) {
			$pos = strpos($this->headers['content-type'],"charset");
			if ($pos !== false) {
				$endpos = strpos($this->headers['content-type'],";",$pos);
				if ($endpos === false) {
					$charset = substr($this->headers['content-type'],$pos);
				} else {
					$length = $endpos - $pos;
					$charset = substr($this->headers['content-type'],$pos,$length);
				}
				list(,$this->encoding) = explode("=",$charset,2);
			}
		}
	}
	
	final static function buildURL($data) {
		$url = "";
		
		if (!isset($data['scheme'])) $data['scheme'] = "http";
		
		$url = $data['scheme']."://";
		if (isset($data['user'])) {
			$url .= $data['user'].(isset($data['pass']) ? ":".$data['pass'] : "")."@";
		}
		$url .= $data['host'];
		if (isset($data['port'])) {
			$url .= ":".$data['port'];
		}
		if (isset($data['path'])) {
			$url .= $data['path'];
		} else {
			$url .= "/";
		}
		if (isset($data['query'])) {
			$url .= "?".$data['query'];
		}
		if (isset($data['fragment'])) {
			$url .= "#".$data['fragment'];
		}
		
		return $url;
	}
	
	final static function fixURL($base,$link) {
		if (!preg_match('~^(http://[^/?#]+)?([^?#]*)?(\?[^#]*)?(#.*)?$~i', $link.'#', $matchesLink)) {
			return false;
		}
		if (!empty($matchesLink[1])) {
			return $link;
		}
		if (!preg_match('~^(http://)?([^/?#]+)(/[^?#]*)?(\?[^#]*)?(#.*)?$~i', $base.'#', $matchesBase)) {
			return false;
		}
		if (empty($matchesLink[2])) {
			if (empty($matchesLink[3])) {
				return 'http://'.$matchesBase[2].$matchesBase[3].$matchesBase[4];;
			}
			return 'http://'.$matchesBase[2].$matchesBase[3].$matchesLink[3];
		}
		$pathLink = explode('/', $matchesLink[2]);
		if ($pathLink[0] == '') {
			return 'http://'.$matchesBase[2].$matchesLink[2].$matchesLink[3];
		}
		$pathBase = explode('/', preg_replace('~^/~', '', $matchesBase[3]));
		if (sizeOf($pathBase) > 0) {
			array_pop($pathBase);
		}
		foreach ($pathLink as $p) {
			if ($p == '.') {
				continue;
			} elseif ($p == '..') {
				if (sizeOf($pathBase) > 0) {
					array_pop($pathBase);
				}
			} else {
				array_push($pathBase, $p);
			}
		}
		return 'http://'.$matchesBase[2].'/'.implode('/', $pathBase).$matchesLink[3];
	}
}

?>