<?php

class MailConfirmer {
	
	public static $links_regexp = "#(https?://[^\s\"'<>]+)#i";
	private $connection_string;
	private $login;
	private $password;
	private $folder;
	private $imap;
	
	public function __construct($server,$port,$login,$password,$driver='imap',$ssl=true,$folder='INBOX') {
		$this->connection_string = '{'.$server.':'.$port.'/'.$driver.($ssl ? '/ssl' : '').'}'.$folder;
		$this->login = $login;
		$this->password = $password;
		$this->folder = $folder;
	}
	
	public function confirm() {
		$mails = $this->getMail();
		$confirmed = array();
		foreach ($mails as $uid => $mail) {
			$links = $this->getLinks($mail['message']);
			$http = new http;
			foreach ($links as $link) {
				$http->GET($link);
				$confirmed[] = $link;
			}
			$this->deleteMail($uid);
		}
		return $confirmed;
	}
	
	public static function getLinks($text) {
		$links = array();
		preg_match_all(self::$links_regexp,$text,$match);
		foreach ($match[1] as $url) {
			$url = str_ireplace('&amp;','&',$url);
			$links[] = $url;
		}
		$links = array_values(array_unique($links));
		return $links;
	}
	
	private function connect() {
		$this->imap = imap_open($this->connection_string, $this->login, $this->password);
	}
	
	private function getMail() {
		$this->connect();
		if (!is_null($this->imap)) {
			$messages = array();
			$mails = imap_fetch_overview($this->imap,'1:'.imap_num_msg($this->imap));
			foreach ($mails as $mail) {
				$title = imap_utf8($mail->subject);
				$struct = imap_fetchstructure($this->imap,$mail->uid,FT_UID);
				$message = "";
				$params = array();
				if ($struct->type == 0) {
					foreach ($struct->parameters as $param) {
						$params[strtolower($param->attribute)] = $param->value;
					}
					$text = imap_fetchbody($this->imap,$mail->uid,1,FT_UID);
					if ($struct->encoding == 3) {
						$text = base64_decode($text);
					}
					if (isset($params['charset'])) {
						$text = iconv($params['charset'],"UTF-8//IGNORE",$text);
					}
					$message = $text;
				} else if ($struct->type == 1) {
					foreach ($struct->parts as $part_number => $part) {
						foreach ($part->parameters as $param) {
							$params[strtolower($param->attribute)] = $param->value;
						}
						$text = imap_fetchbody($this->imap,$mail->uid,$part_number+1,FT_UID);
						if ($struct->encoding == 3) {
							$text = base64_decode($text);
						}
						if (isset($params['charset'])) {
							$text = iconv($params['charset'],"UTF-8//IGNORE",$text);
						}
						$message .= $text."\r\n";
					}
				}
				$messages[$mail->uid] = array(
					'title' => $title,
					'message' => $message
				);
			}
			imap_close($this->imap);
			return $messages;
		} else {
			return false;
		}
	}
	
	private function deleteMail($uid) {
		return imap_delete($this->imap,$uid,FT_UID);
	}
}

?>